******************************************************************************
*
*       Function:   GET_QUEUE_PARMS
*       Usage:      call get_queue_parms (stream)
*       Purpose:    This routine is called during the start_stream phase of
*                   a queue.  It initiates the reading of configuration
*                   information for the queue associated with the stream.
*       Output:     The appropriate streams(stream) fields are filled with
*                   any new values specified in the configuration file.
*
******************************************************************************

        subroutine get_queue_parms (stream)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer stream, unitno, len, idx
            character*(LINELEN) line
            character*(OPTLEN) option
            logical done
            external set_one
            logical set_one, strcmp
            external blength, strcmp
            integer blength

*           Get the configuration file.

            unitno = 20+stream
            open (UNIT=unitno, FILE=PARMSFILE, SHARED, STATUS='OLD')

*           Loop until we find the configuration line we are looking for.

            done = .false.
            do while (.not. done)

*               Get a line from the file.

                read (unitno, 3, end=10) line
  3             format (a80)

*               Find a non comment, non continuation line.

                if ((line(1:1) .ne. ' ').and.(line(1:1) .ne. '!')) then
                    len = blength(streams(stream).exec_queue)+1

*                   Check for proper queue name

                    done =
     .                  strcmp (line(1:len),streams(stream).exec_queue(1:len))
                endif
            end do

*           Once we have found an entry, parse it.

            call parse_options (line, len, unitno, streams(stream), set_one)

   10       continue
            close (UNIT=unitno)
        end

******************************************************************************
*
*       Function:   SET_ONE
*       Usage:      success = set_one (conf_name, strval, strlen, intval,
*                                      boolval, stream_id)
*       Purpose:    This routine is incharge of recognizing 'name' as a
*                   valid configuration parameter, and then using one of the
*                   values passed to establish the particular configuration
*                   value.  'stream' is the streams structure that is being
*                   configured.
*       Output:     A configuration parameter is established.
*
******************************************************************************

        logical function set_one (name, strval, strlen, intval, boolval, stream)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'
            include '($pqldef)'

            character*(*) name, strval
            integer strlen, intval, dummy, stat
            logical boolval, strcmp
            external strcmp, getuinfo, add_quota
            record /stream_info/ stream

            set_one = .true.
            if (strcmp (name, 'quepri')) then
                stream.proc_pri = strval(1:strlen)
            else if (strcmp (name, 'file')) then
                stream.proc_spec = strval(1:strlen)
            else if (strcmp (name, 'pause')) then
                stream.pause_time = intval
            else

*               Otherwise, this queue requires a special process that MUST
*               be dedicated to this queue, so we signify that fact, and
*               then check the rest of the items.

                stream.special = .true.

                if (strcmp (name, 'output')) then
                    stream.outfile = strval(1:strlen)
                else if (strcmp (name, 'error')) then
                    stream.errfile = strval(1:strlen)
                else if (strcmp (name, 'expire')) then
                    stream.expire = intval
                else if (strcmp (name, 'detached')) then
                    stream.detached = boolval
                else if (strcmp (name, 'astlm')) then
                    call add_quota (stream, PQL$_ASTLM, intval)
                else if (strcmp (name, 'biolm')) then
                    call add_quota (stream, PQL$_BIOLM, intval)
                else if (strcmp (name, 'bytlm')) then
                    call add_quota (stream, PQL$_BYTLM, intval)
                else if (strcmp (name, 'cpulm')) then
                    call add_quota (stream, PQL$_CPULM, intval)
                else if (strcmp (name, 'diolm')) then
                    call add_quota (stream, PQL$_DIOLM, intval)
                else if (strcmp (name, 'enqlm')) then
                    call add_quota (stream, PQL$_ENQLM, intval)
                else if (strcmp (name, 'fillm')) then
                    call add_quota (stream, PQL$_FILLM, intval)
                else if (strcmp (name, 'jtquota')) then
                    call add_quota (stream, PQL$_JTQUOTA, intval)
                else if (strcmp (name, 'pgflquota')) then
                    call add_quota (stream, PQL$_PGFLQUOTA, intval)
                else if (strcmp (name, 'prclm')) then
                    call add_quota (stream, PQL$_PRCLM, intval)
                else if (strcmp (name, 'tqelm')) then
                    call add_quota (stream, PQL$_TQELM, intval)
                else if (strcmp (name, 'wsdefault')) then
                    call add_quota (stream, PQL$_WSDEFAULT, intval)
                else if (strcmp (name, 'wsextent')) then
                    call add_quota (stream, PQL$_WSEXTENT, intval)
                else if (strcmp (name, 'wsquota')) then
                    call add_quota (stream, PQL$_WSQUOTA, intval)
                else if (strcmp (name, 'user')) then
                    stream.privs(1) = 0
                    stream.privs(2) = 0

*                   If USER=USER is used, then signal to use the submitting
*                   user's info.

                    if (strcmp (strval(1:strlen), 'user')) then
                        stream.use_sub_user = .true.
                    else
                        stream.user = strval(1:strlen)

                        call getuinfo (stream)
                    endif
                else
                    set_one = .false.
                    stream.special = .false.
                endif               
            endif
        end

*
*       Add a quota item to the list of quotas for the process create for
*       use by this stream.
*
        subroutine add_quota (stream, which, value)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'
            include '($pqldef)'

            record /stream_info/ stream
            integer which, value

            stream.quota_idx = stream.quota_idx + 1
            stream.quotas(stream.quota_idx).type = which
            stream.quotas(stream.quota_idx).value = value
            stream.quotas(stream.quota_idx+1).type = PQL$_LISTEND
        end
