******************************************************************************
*
*       Function:   HANDLER
*       Usage:      Called by the system at exception time.
*       Purpose:    To handle internal errors/exceptions that occur during
*                   execution of the symbiont.  This code will log the
*                   error to the log file, and the print operators consoles
*                   using the put_line() function, and sys$putmsg.
*       Output:     A log file entry, and console message.
*
******************************************************************************

        integer function handler (sigargs, mechargs)
            implicit none
            include 'defs.inc'

            integer sigargs(*), mechargs(*), newargs(1:10), i
            external put_line, sys$putmsg

            newargs(1) = 10
            do i=1, sigargs(1) - 2
                newargs(1+i) = sigargs(i+1)
            end do

            do i=sigargs(1), 10
                newargs(i) = 0
            end do

            call sys$putmsg (newargs, put_line, )

            handler = SS$_RESIGNAL
        end

******************************************************************************
*
*       Function:   PUT_LINE
*       Usage:      Called from sys$putmsg at condition signalling time.
*       Purpose:    This routine is used as an output routine for sys$putmsg.
*                   It uses log_sys() to report the string passed from
*                   sys$putmsg.
*       Output:     A log file message, and an operator message.
*
******************************************************************************

        logical function put_line (line)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            character *(*) line
            external log_sys

            call log_sys (CTLSMB__SIGNAL, line)
            put_line = .false.
        end
