******************************************************************************
*
*       Function:   LOG_STR
*       Usage:      call log_str (vms_status, string)
*       Purpose:    This routine is similiar to log_err, only it allows a
*                   allows a string to be passed along with the status to
*                   further clarify a status value.
*       Output:     A message is logged to the log file which includes the
*                   message text of the status passed, and the string that
*                   is passed as well.
*
******************************************************************************

        subroutine log_str (status, str)

            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer status, stat, iostat, blength
            integer*2 len, blen
            character*(*) str
            character*(BUFSIZ) msg, buf
            character*(24) timbuf
            external open_errlog, close_errlog, blength, signal

            timbuf = ' '

            call signal (sys$asctim (,%descr(timbuf),,))

            call signal (sys$getmsg (%val (%loc (CTLSMB__INFO)),
     .                               %ref (len), %descr (msg),,))

            call signal (sys$fao (%descr(msg(1:len)), %ref (blen), %descr (buf),
     .              %descr (streams(last_stream).queue(
     .                      1:blength(streams(last_stream).queue))),
     .              %descr (procs(last_idx).job_name(
     .              1:blength(procs(last_idx).job_name))),
     .              %val (procs(last_idx).entry)))

            buf(1:1) = '-'

            call signal (sys$getmsg (%val (%loc (status)),
     .                                      %ref (len), %descr (msg),,))
            msg(1:1) = '-'

            call open_errlog (iostat)

            if (iostat .ne. 0) then
                call sys$exit (%val (%loc (status)))
            endif

            write (ERRUNIT, *) '%CTLSMB-I-TIME '//timbuf
            write (ERRUNIT, *) buf(1:blen)
            write (ERRUNIT, *) '-CTLSMB-I-TEXT '//str
            write (ERRUNIT, *) msg(1:len)
            write (ERRUNIT, *) '*'

            call close_errlog (iostat)
        end
