******************************************************************************
*
*       Function:   PARSE_OPTIONS
*       Usage:      call parse_options (buffer, buf_idx, io_unit,
*                                       option_arg, set_func)
*       Purpose:    This routine provides a genalized options file parsing
*                   facility.  It accepts as input an initial input line
*                   which may contain some information, an index into that
*                   line which indicates the start of the usable information,
*                   a FORTRAN unit number where additional lines can be
*                   read from, a qualifying argument that will help the
*                   final argument, a function to set each parameter, to
*                   distinguish which thing is being configured.
*       Output:     The parsed options are established.
*
******************************************************************************

        subroutine parse_options (line, idx, unitno, option_arg, set_options)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            character*(*) line
            integer idx, unitno, i, iostat, intval, strlen, rval
            integer option_arg
            character*(OPTLEN) name, strval
            logical done, next, boolval
            external set_options, skip_white, blength, parse_one
            integer skip_white, blength
            logical set_options, parse_one

            done = .false.
            do while (.not. done)
                rval = skip_white (line, idx)
                done = (rval .eq. AT_EOE)
                if (.not. done) then
                    do while (rval .eq. AT_CONT)
                        next = .true.
                        do while (next)
                            read (unitno, 10, iostat=iostat) line
  10                        format (a80)
                            next = (iostat .eq. 0)
                            if (next) then

*                               Allow comments between lines of continued
*                               parameters.

                                next = (line(1:1) .eq. '!')
                            else
                                done = .true.
                                goto 20
                            endif
                        end do
                        idx = 1
                        rval = skip_white (line, idx)
                    end do
                    done = (rval .eq. AT_EOE)
                    if (.not. done) then
                        boolval = .false.
                        intval = -1
                        strlen = 0
                        if (parse_one (line, idx, name, strval,
     .                                          strlen, intval, boolval)) then
                            done = .not. set_options (name(1:blength(name)),
     .                          strval, strlen, intval, boolval, option_arg)
                        endif
                    endif
                endif
 20             continue
            end do
        end

******************************************************************************
*
*
*
*
******************************************************************************

        logical function parse_one (line, idx, name, retstr,
     .                                              retlen, retint, retbool)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'
            parameter GET_NAME=1
            parameter GET_VALUE=2

            character*(*) line, name, retstr
            character*(OPTLEN) value
            integer idx, retlen, retint, i, state
            logical retbool
            logical*1 done, done_value, seen_eq, in_quotes
            external strcmp, atoi
            integer atoi
            logical strcmp
            character ch

            parse_one = .true.

            i = 1
            state = GET_NAME
            done_value = .false.
            seen_eq = .false.
            done = .false.
            name = ' '
            value = ' '

            do while (.not. done)
                ch = line(idx:idx)
                if (((ch .eq. ',').or.(ch .eq. ' ')).and.(.not. in_quotes)) then
                    done = .true.
                else if (ch .eq. '=') then
                    state = GET_VALUE
                    i = 1
                    seen_eq = .true.
                else
                    if (state .eq. GET_NAME) then
                        name(i:i) = ch
                        i = i + 1
                    else if (state .eq. GET_VALUE) then
                        if (ch .eq. '"') in_quotes = .not. in_quotes
                        value(i:i) = ch
                        i = i + 1
                        done_value = .true.
                    endif
                endif

*               Must do this to skip over comma

                idx = idx + 1
            end do

            retlen = i-1
            retstr = value(1:retlen)

            if (seen_eq .and. done_value) then
                retint = atoi (value(1:retlen))
            else if (seen_eq) then
                parse_one = .false.
            else
                if (strcmp (value(1:3), 'no_')) then

*                   Trim off the "no_" qualifier

                    value = value (4:)
                    retbool = .false.
                else
                    retbool = .true.
                endif
            endif
        end


*
*       Skip to the next qualifier, or return End Of Entry, or AT CONTinuation.
*

        integer function skip_white (str, idx)
            implicit none
            include 'defs.inc'

            character*(*) str
            integer idx
            logical done

            if ((idx .eq. 1).and. (str(1:1) .ne. ' ')) then
                skip_white = AT_EOE
            else
                done = .false.
                do while (.not.done)
                    done = (str(idx:idx) .ne. ' ')
                    if (.not. done) done = (idx+1 .gt. len (str))
                    if (.not. done) idx = idx + 1
                end do
                if (idx+1 .gt. len(str)) then
                    skip_white = AT_CONT
                else
                    skip_white = AT_PARM
                endif
            endif
        end
