******************************************************************************
*
*       Function:   PROC_DELETED
*       Usage:      Called as an AST service routine.
*       Purpose:    This routine is called as an AST service routine when
*                   a read on the termination mailbox associated with
*                   a specific queue completes, i.e. the process is deleted.
*                   The death of a process may or may not be expected, that
*                   is determined by the value of proc.dying.  If proc.dying
*                   is true, then the process was deleted by the symbiont.
*                   Otherwise, the death was not expected, and something else
*                   caused the process to be deleted.
*       Output:     If the process deletion was not expected, then
*                   report_deleted() will issue a message to the log file,
*                   and to the OPERATOR consoles.  Otherwise, there is no
*                   output.
*
******************************************************************************

        integer function proc_deleted (account)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'
            include '($accdef)'

            integer stat

*           Use an array so that the size of the structure can be computed.

            record /terminate/ account

            external report_deleted, get_proc_del
            integer get_proc_del, idx

*           If there was an accounting record read, then process it.

            idx = account.proc_idx
            if ((procs(idx).delprciosb.count .eq. ACC$K_TERMLEN) .and.
     .                          ((procs(idx).delprciosb.cond.and.1).eq.1)) then
                call report_deleted (account.acc.pid, account.acc.estat,
     .                               procs(idx).stream)
            else
                call log_str (%val (procs(idx).delprciosb.cond),
     .                       'Invalid accounting record for deleted process')
            endif
        end                          
