******************************************************************************
*
*       Function:   REPORT_DELETED
*       Usage:      call report_deleted (VMS_pid, VMS_status, SMB_stream_id)
*       Purpose:    This routine is used to process the information associated
*                   with a deleted process.  This routine checks to see that
*                   the deletion was intentional, and if it wasn't, then an
*                   error message is issued.
*       Output:     A possible error message, and an available process slot.
*
******************************************************************************

        subroutine report_deleted (pid, status, stream)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'
            
            integer pid, status, stream, i, stat
            external ex_stat, sys$cantim, reply2, reply, sys$dassgn, sys$delmbx
            integer sys$cantim, sys$dassgn, sys$delmbx, sys$setef, sys$cancel
            logical done
            external sys$setef, sys$cancel

            done = .false.
            i = 1
            do while ((i.le. MAXPROCS) .and. (.not.done))
                if ((procs(i).started) .and. (procs(i).pid .eq. pid)) then

*                   Cancel any previous timer request.

                    stat = sys$cantim (procs(i),)

*                   Delete and deassign the mailbox channel

                    call signal (sys$cancel (%val(procs(i).mailchan)))
                    call signal (sys$delmbx (%val(procs(i).mailchan)))
                    call signal (sys$dassgn (%val(procs(i).mailchan)))

                    call signal (sys$cancel (%val(procs(i).delprchan)))
                    call signal (sys$delmbx (%val(procs(i).delprchan)))
                    call signal (sys$dassgn (%val(procs(i).delprchan)))

*                   Tell the JBC that the task is gone!

                    if ((procs(i).starting .or. procs(i).ready)
     .                                      .and. (.not. procs(i).dying)) then

                        if (status .eq. 0) status = SS$_ABORT

                        call reply2 (JBC$_JOBABORT, status,
     .                                          SMBMSG$K_TASK_COMPLETE, stream)
                    else if ((.not. procs(i).dying) .and. procs(i).avail) then
                        call ex_stat ('Unexpected subprocess termination',
     .                      pid, status, stream)
                    endif


*                   Set the slot back up so that it can be reused.

                    procs(i).avail = .true.
                    procs(i).started = .false.
                    procs(i).starting = .false.
                    procs(i).ready = .false.
                    procs(i).dying = .false.
                    procs(i).dedicated = .false.

*                   Free the event flag

                    stat = lib$free_ef (procs(i).efn)
                    if (procs(i).alloc_ef .ne. -1) then
                        call signal (sys$setef (%val (procs(i).alloc_ef)))
                    endif
                    done = .true.
                endif
                i = i + 1
            end do
        end 
        
******************************************************************************
*
*       Function:   EX_STAT
*       Usage:      call ex_stat ('text', VMS_pid, VMS_status, SMB_stream_id)
*       Purpose:    This routine is used to issue a message about the status
*                   of the process whose pid is passed.
*       Output:     An error message in the log and to the operators console.
*
******************************************************************************

        subroutine ex_stat (mess, pid, status, stream)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            character*(*) mess
            integer pid, status, stream, iostat
            integer*2 len
            character*512 buf

            external signal

            call signal (sys$fao ('!AS, pid=!XL', %ref(len),
     .                      %descr (buf),
     .                      mess,
     .                      %val(pid)))

            if (status.eq.0) status = SS$_ABORT
            call log_sys (%val (status), buf(1:len))
        end 
