******************************************************************************
*
*       Function:   SMB$MAIN
*       Usage:      Main program for symbiont
*       Purpose:    This routine is the main driver for the symbiont.  It sits
*                   idle in a loop waiting for work, and dispatches the
*                   routine process() whenever it has some.
*       Output:     NONE
*
******************************************************************************

        implicit none
        include 'defs.inc'
        include 'struct.inc'
        include 'extern.inc'
        include 'global.inc'
        include '($jpidef)'

        integer stat, i
        integer*4 privs(1:2), prevprivs(1:2)
        external signal, wake_proc, process, handler, sys$setprv
        character qual(1:4)
        integer strm
        equivalence (strm, qual)

*       Set the no work pending flag prior to calling smb$initialize
*       because there might be an immediate AST delivery if work is
*       in a stopped queue.

        work_pending = 0

*       Establish a condition handler.

        call lib$establish (handler)

*       All privs on.

        privs(1) = -1
        privs(2) = -1
        call sys$setprv (%val(1),privs,,prevprivs)

        cur_privs(1) = -1
        cur_privs(2) = -1
        call signal (lib$getjpi (JPI$_UIC,,,cur_uic,,))

*       Create a new log file.

        open (unit=ERRUNIT, status='NEW', file=ERRFILE, recl=30000)
        close (unit=ERRUNIT)

*       Initialize the dummy block used for reporting conditions about
*       idle subprocesses.

        streams(MAX_STREAMS+1).queue = 'SYMBIONT'
        streams(MAX_STREAMS+1).exec_queue = 'SYMBIONT'

        last_stream = MAX_STREAMS+1
        last_idx = 0

*       Currently no streams are started.

        num_streams = 0

*       Initialize all of the subprocess slots.

        do i=1,MAXPROCS

*           Slot is available

            procs(i).avail = .true.

*           Process is not running

            procs(i).started = .false.
            procs(i).starting = .false.

*           Process has not read from SYS$INPUT

            procs(i).ready = .false.

*           We have not requested the process to exit.

            procs(i).dying = .false.

*           Given a proc structure, it would be nice to know who owns
*           the process, so put that info in as well.

            procs(i).idx = i
            procs(i).accnt.proc_idx = i

*			No stream is waiting on a process.

			procs(i).alloc_ef = -1
        end do

*       Tell the JBC that we are ready to go.

        call signal (smb$initialize (SMBMSG$K_STRUCTURE_LEVEL, wake_proc,
     .                                                      MAX_STREAMS))

*       Loop forever waiting to be woke up when there is work to be done.
*       work_pending is a counter that is incremented by wake_proc, for
*       each AST that is delivered to it.  The code below is used to work
*       around the fact (bug or feature) that $WAKE/$HIBER combinations
*       allow only one $HIBER to be executed no matter how many $WAKE's
*       have been previously issued.

        do while (.true.)
            if (work_pending .lt. 2) then
                call sys$hiber
            endif
            work_pending = work_pending - 1
            if (work_pending .ge. 0) then
*                call sys$setast (%val(0))
                call process
*                call sys$setast (%val(1))
            endif
        end do

        end

****************************************************************************
*
*   Called as an AST service routine when there is work to be done.  We
*   wake up the process, increment the work_pending counter, and return.
*
****************************************************************************

        integer function wake_proc (dummy)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer dummy

            work_pending = work_pending + 1
            call sys$wake (,)
        end
