******************************************************************************
*
*       Function:   START_PROC
*       Usage:      call start_proc (proc_structure, SMB_stream_id, VMS_status)
*       Purpose:    This routine is used to start a new subprocess when one
*                   is needed.  It creates an I/O mailbox and channel for
*                   SYS$INPUT of the subprocess.  Then, a subprocess is
*                   started, running DCL.  We ask for notification of the
*                   process deletion by specifying a termination mailbox
*                   in the $CREPRC call.
*       Output:     A running process, ready for our use.
*
******************************************************************************

        logical function start_proc (proc, cstrm, lstat)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'     
            include '($prcdef)'
            include '($syidef)'
            include '($jpidef)'

            record /sub_proc/ proc

            integer cstrm, i, lstat, stat

            character*20 proc_nam

            integer get_proc_del, setuic, flags, prio, pid

            external get_proc_del, sys$cantim, blength, log_str, prcnam
            external signal, proc_ready, proc_deleted, lib$getsyi
            external sys$dassgn, setuic, atoi
            character*2 prcnam
            integer sys$cantim, blength, sys$dassgn, atoi
            external sys$delmbx, sys$cancel
            integer sys$delmbx, sys$cancel

*           Initialize some values  

*           Cancel any previous timer request before checking the started
*           flag.  Certain race conditions could otherwise cause the process
*           to be deleted right out from underneath us, because of process
*           expiration.

            stat = sys$cantim (proc,)

*           Set the starting flag.

            proc.starting = .true.
            start_proc = .false.
            lstat = SS$_NORMAL

            if (proc.started) then
                stat = lib$getjpi (JPI$_UIC, proc.pid,,pid,,)
                if ((stat.and.1).ne.1) then
                    call log_str (%val (stat),
     .                          'Started process is gone in start_proc')
                    proc.started = .false.
                    call signal (sys$cancel (%val(proc.mailchan)))
                    call signal (sys$delmbx (%val(proc.mailchan)))
                    call signal (sys$dassgn (%val(proc.mailchan)))

                    call signal (sys$cancel (%val(proc.delprchan)))
                    call signal (sys$delmbx (%val(proc.delprchan)))
                    call signal (sys$dassgn (%val(proc.delprchan)))

                    stat = lib$free_ef (proc.efn)
                    proc.dedicated = .false.
                    proc.ready = .false.
                endif
            endif

*           If the subprocess is not started, then start one.

            if (.not. proc.started) then

*               Get an event flag

                call signal (lib$get_ef (proc.efn))

*               Create a process name         

                proc_nam = 'CTLSMB_' // prcnam(proc.idx)
                     
*               Create the mailbox name

                proc.mboxnam = 'CTLSMB$MBX_' // prcnam(proc.idx)

*               Create a subprocess to use

                call lib$getsyi (SYI$_DEFPRI, prio, , , , )
                                                    
                proc.started = .true.             
                proc.uic = streams(cstrm).uic
                if (streams(cstrm).special) then
                    proc.dedicated = .true.
                    prio = atoi (streams(cstrm).proc_pri)
                endif

                flags = 0
                if (streams(cstrm).detached) then
                    proc.detached = .true.
                    flags = PRC$M_DETACH
                endif

                proc.delprcmbx = DELPRCNAM // prcnam(proc.idx)

*               If our UIC matches that of the process to be created then
*               just do the stuff to create a mailbox and such.

                if (streams(cstrm).uic .eq. cur_uic) then

*                   Create a process deleted accounting message mailbox

                    call signal (sys$crembx (
     .                     ,
     .                     proc.delprchan,
     .                     ,         
     .                     ,
     .                     %val(255*256),
     .                     ,    
     .                     %descr(proc.delprcmbx(1:
     .                          blength(proc.delprcmbx)))))
                                             
*                   Get the mailbox number for $CREPRC to use for termination
*                   reports.
                                       
                    call signal (lib$getdvi (DVI$_UNIT,
     .                          proc.delprchan,, proc.delprcmbxno, , ))

*                   Create a communications mailbox

                    call signal (sys$crembx (
     .                  %val(1),     
     .                  proc.mailchan,
     .                  , ,
     .                  %val(255*256),
     .                  ,
     .                  %descr(proc.mboxnam(1:13))))

                    lstat = sys$creprc (
     .                  proc.pid,
     .                  DCL_IMAGE,
     .                  %descr(proc.mboxnam(1:13)),
     .                  streams(cstrm).outfile,
     .                  streams(cstrm).errfile,
     .                  streams(cstrm).privs,
     .                  streams(cstrm).quotas,
     .                  proc_nam(1:9),
     .                  %val(prio), 
     .                  ,                                           
     .                  %val(proc.delprcmbxno), %val(flags)
     .                  )
                else

*                   Otherwise, we must change our UIC and USERNAME,
*                   and then create the mailbox so that it is owned by
*                   the reader of the mailbox (who might not be privileged).

                    call signal (setuic (%descr (streams(cstrm).user),
     .                                          %val(streams(cstrm).uic)))
                                                                        
*                   Create a process deleted accounting message mailbox

                    call signal (sys$crembx (
     .                     ,
     .                     proc.delprchan,
     .                     ,         
     .                     ,
     .                     %val(255*256),
     .                     ,    
     .                     %descr(proc.delprcmbx(1:
     .                          blength(proc.delprcmbx)))))
                                             
*                   Get the mailbox number for $CREPRC to use for termination
*                   reports.
                                       
                    call signal (lib$getdvi (DVI$_UNIT,
     .                              proc.delprchan,, proc.delprcmbxno, , ))

*                   Create a communications mailbox

                    call signal (sys$crembx (
     .                  %val(1),         
     .                  proc.mailchan,
     .                  , ,
     .                  %val(255*256),
     .                  ,
     .                  %descr(proc.mboxnam(1:13))))
                                    
                    lstat = sys$creprc (                            
     .                  proc.pid,
     .                  DCL_IMAGE,
     .                  %descr(proc.mboxnam(1:13)),
     .                  streams(cstrm).outfile,
     .                  streams(cstrm).errfile,
     .                  streams(cstrm).privs,
     .                  streams(cstrm).quotas,
     .                  proc_nam(1:9),
     .                  %val(prio),
     .                  ,
     .                  %val(proc.delprcmbxno), %val(flags)
     .                  )
                        
                    call signal (setuic ('SYSTEM              ', %val(cur_uic)))
                endif       
     
                if ((lstat.and.1) .ne. 1) then
                    proc.started = .false.
                else

*                   If the $CREPRC works, then ask for a WRTATTN ast when
*                   the process completes.  NOTE:  It never should unless
*                   we $DELPRC it.

                    lstat = get_proc_del (cstrm, proc)
                endif               
            else if (proc.ready) then

*               This is an internal error condition because a started process
*               should never be ready.

                lstat = %loc (CTLSMB__PROCREADY)
            endif

*           If we have a process then do the rest of the work.

            if ((lstat.and.1) .eq. 1) then
                start_proc = .true.
            endif
        end

*   Create a hexidecimal 2 digit ASCII character representation of
*   the number passed.

        character*2 function prcnam (idx)
            implicit none
            integer*4 high, low, idx
            character h_char(1:4),l_char(1:4)
            equivalence (h_char, high)
            equivalence (l_char, low)

*           Create a 2 character hex number for the process identfier

            high = idx / 16
            low = (idx .and. 15)

            if (low .le. 9) then
                low = low + 48
            else
                low = low + 55
            endif

            if (high .le. 9) then
                high = high + 48
            else
                high = high + 55
            endif

            prcnam = h_char(1)//l_char(1)
        end
